@@ -1,7 +1,5 @@
-try:
-    from hashlib import sha1
-except ImportError:
-    from sha import new as sha1
+import os
+from binascii import hexlify
 import time
 from datetime import datetime, timedelta
 
@@ -9,7 +7,10 @@ from django.db import models
 from django.conf import settings
 from django.utils import simplejson
 from django.contrib.auth.models import User
+from django.db.utils import IntegrityError
 
+class URLAuthError(Exception):
+    pass
 
 class AuthKeyManager(models.Manager):
 
@@ -27,11 +28,21 @@ class AuthKeyManager(models.Manager):
 
         if 'onetime' in kwargs:
             key.onetime = kwargs.pop('onetime')
-
-        source = '%s%d%d' % (settings.SECRET_KEY, time.time(), id({}))
-        key.id = sha1(source).hexdigest()
         key.data = simplejson.dumps(kwargs)
-        key.save()
+
+        # Try 10 times to create AuthKey instance with unique PK
+        for x in xrange(10):
+            key.pk = hexlify(os.urandom(20))
+            try:
+                key.save(force_insert=True)
+            except IntegrityError:
+                key.pk = None
+            else:
+                break
+
+        if not key.pk:
+            raise URLAuthError('Could not create unique key')
+
         return key.id
 
     def wrap_url(self, url, **kwargs):
